/*****************************************************************************\
*  CANpie                                                                     *
*                                                                             *
*  File        : canpie.h                                                     *
*  Description : Definitions / Constants for CANpie                           *
*  Author      : Uwe Koppe                                                    *
*  e-mail      : koppe@microcontrol.net                                       *
*                                                                             *
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
*                                                                             *
*   This program is free software; you can redistribute it and/or modify      *
*   it under the terms of the GNU General Public License as published by      *
*   the Free Software Foundation; either version 2 of the License, or         *
*   (at your option) any later version.                                       *
*                                                                             *
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
*                                                                             *
*  History                                                                    *
*  Vers.    Date        Comment                                         Aut.  *
*  -----    ----------  ---------------------------------------------   ----  *
*  0.1      04.12.1998  Initial version                                 UK    *
*  0.2      29.04.1999  Changed structures, new data type definitions   UK    *
*  0.3                  - no changes -                                        *
*  0.4                  - no changes -                                        *
*  0.5                  - no changes -                                        *
*  0.6      15.06.2000  moved defintions from cpmsg.h                   UK    *
*                       set to CANpie release 0.6                             *
*	0.7		06.11.2000	added new error codes									UK		*
*								added missing buffer number									*
*	0.8		15.02.2001	set to CANpie release 0.8								UK		*
*                                                                             *
\*****************************************************************************/


#ifndef  _CANPIE_H_
#define  _CANPIE_H_

//------------------------------------------------------------------------------
// CVS version information:
// $Id: canpie.h,v 1.4 2005/07/13 11:56:43 microcontrol Exp $
//------------------------------------------------------------------------------


//-----------------------------------------------------------------------------
/*!   \file    canpie.h
**    \brief   CANpie constants, structures and enumerations
**
**    This file holds constants and structures used within CANpie.
**
*/


/*----------------------------------------------------------------------------*\
** Include files                                                              **
**                                                                            **
\*----------------------------------------------------------------------------*/

#include "cp_arch.h"       // Architecture dependent definitions


/*----------------------------------------------------------------------------*\
** Definitions & Enumerations                                                 **
**                                                                            **
\*----------------------------------------------------------------------------*/


#ifndef  CP_CAN_MSG_TIME
#define  CP_CAN_MSG_TIME            0
#endif

#ifndef  CP_CAN_MSG_USER
#define  CP_CAN_MSG_USER            0
#endif

#ifndef  CP_CAN_MSG_MACRO
#define  CP_CAN_MSG_MACRO           0
#endif

#ifndef  CP_CHANNEL_MAX
#define  CP_CHANNEL_MAX             1
#endif

#define  CP_VERSION_MAJOR           2
#define  CP_VERSION_MINOR           0


/*-----------------------------------------------------------------------------
** CP_MASK_XXX
** mask for CAN message (RTR, Extended CAN)
*/


/*-------------------------------------------------------------------*/
/*!
** \def  CP_MASK_STD_FRAME
**
** Mask for standard frame (11 bits)
*/
#define  CP_MASK_STD_FRAME 0x000007FF


/*-------------------------------------------------------------------*/
/*!
** \def  CP_MASK_EXT_FRAME
**
** Mask for extended frame (29 bits)
*/
#define  CP_MASK_EXT_FRAME 0x1FFFFFFF


/*-------------------------------------------------------------------*/
/*!
** \def  CP_MASK_EXT_BIT
**
** Set the EXT bit in the v_MsgId field
*/
#define  CP_MASK_EXT_BIT   0x80000000


/*-------------------------------------------------------------------*/
/*!
** \def  CP_MASK_RTR_BIT
**
** Set the RTR bit in the v_MsgId field
*/
#define  CP_MASK_RTR_BIT   0x40000000


/*-------------------------------------------------------------------*/
/*!
** \def  CP_MASK_DLC_BITS
**
** Mask for DLC field in v_MsgFlags
*/
#define  CP_MASK_DLC_BITS  0x0000000F


/*-------------------------------------------------------------------*/
/*!
** \def  CP_MASK_BUF_BITS
**
** Mask for buffer field in v_MsgFlags
*/
#define  CP_MASK_BUF_BITS  0x0000FF00

/*----------------------------------------------------------------------------*/
/*!
** \enum    CpErr
** \brief   CANpie Error codes
**
**
*/
enum CpErr {

   /*!   No error (00dec / 00hex)
   */
   CpErr_OK = 0,

   /*!   Error not specified (01dec / 01hex)
   */
   CpErr_GENERIC,

   /*!   Hardware failure (02dec / 02hex)
   */
   CpErr_HARDWARE,

   /*!   Initialisation failure (03dec / 03hex)
   */
   CpErr_INIT_FAIL,

   /*!   Channel is initialized, ready to run (04dec / 04hex)
   */
   CpErr_INIT_READY,

   /*!    CAN channel was not initialized (05dec / 05hex)
   */
   CpErr_INIT_MISSING,

   /*!   Receive buffer is empty (05dec / 05hex)
   */
	CpErr_RCV_EMPTY,

	/*!	Receive buffer overrun (06dec / 06hex)
	*/
	CpErr_OVERRUN,

   /*!   Transmit buffer is full (07dec / 07hex)
   */
	CpErr_TRM_FULL,

	/*!   CAN message has wrong format (10dec / 0Ahex)
	*/
   CpErr_CAN_MESSAGE = 10,

   /*!   CAN identifier not valid (11dec / 0Bhex)
   */
   CpErr_CAN_ID,

   /*!   CAN data length code not valid (12dec / 0Chex)
   */
   CpErr_CAN_DLC,

   /*!   FIFO is empty (20dec / 14hex)
   */
   CpErr_FIFO_EMPTY = 20,

   /*!   Message is waiting in FIFO (21dec / 15hex)
   */
   CpErr_FIFO_WAIT,

   /*!   FIFO is full (22dec / 16hex)
   */
   CpErr_FIFO_FULL,

   /*!   FIFO size is out of range (23dec / 17hex)
   */
   CpErr_FIFO_SIZE,

   /*!   Parameter of FIFO function is out of range (24dec / 18hex)
   */
   CpErr_FIFO_PARM,
   
   /*!   Controller is in error passive (30dec / 1Ehex)
   */
   CpErr_BUS_PASSIVE = 30,

   /*!   Controller is in bus off (31dec / 1Fhex)
   */
   CpErr_BUS_OFF,

   /*!   Controller is in warning status (32dec / 20hex)
   */
   CpErr_BUS_WARNING,


   /*!   Channel out of range (40dec / 28hex)
   */
   CpErr_CHANNEL = 40,

   /*!   Register address out of range (41dec / 29hex)
   */
   CpErr_REGISTER,

   /*!   Baudrate out of range (42dec / 2Ahex)
   */
   CpErr_BAUDRATE,

   /*!   Buffer number out of range (43dec / 2Bhex)
   */
   CpErr_BUFFER,
   
   /*!   Parameter number out of range (44dec / 2Chex)
   */
   CpErr_PARAM,

   /*!   Function is not supported (50dec / 32hex)
   */
   CpErr_NOT_SUPPORTED = 50
};


/*----------------------------------------------------------------------------*/
/*!
** \enum    CP_FIFO
** \brief   FIFO Buffer numbers
*/
enum CP_FIFO {
   CP_FIFO_RCV = 0,
   CP_FIFO_TRM
};


/*----------------------------------------------------------------------------*/
/*!
** \enum    CP_CALLBACK
** \brief   Callback Return Codes
**
** These return values are used by the callback functions that can be
** installed by the function CpUserIntFunctions(). <p>
** \b Example
** \code
** _U08 MyCallback(_TsCpCanMsg * pCanMsgV)
** {
**    // Do something with IDs < 100
**    if( CpMacGetStdId(pCanMsgV) < 100)
**    {
**       //.....
**       return(CP_CALLBACK_PROCESSED)
**    }
**
**    // Put all other messages into the FIFO
**    return (CP_CALLBACK_PUSH_FIFO);
** }
** \endcode
** <br>


*/
enum CP_CALLBACK {

   /*!
   ** Message was processed by callback and is not inserted in the FIFO
   */
   CP_CALLBACK_PROCESSED = 0,

   /*!
   ** Message was processed by callback and is inserted in the FIFO
   */
   CP_CALLBACK_PUSH_FIFO
};


/*----------------------------------------------------------------------------*/
/*!
** \enum    CP_BAUD
** \brief   Fixed baudrates
**
** The values of the enumeration CP_BAUD are used as parameter for the
** functions CpUserBaudrate() and CpCoreBaudrate().
*/
enum CP_BAUD {
   /*!
   ** Baudrate 10 kBit/sec
   */
   CP_BAUD_10K = 0,

   /*!
   ** Baudrate 20 kBit/sec
   */
   CP_BAUD_20K,

   /*!
   ** Baudrate 50 kBit/sec
   */
   CP_BAUD_50K,

   /*!
   ** Baudrate 100 kBit/sec
   */
   CP_BAUD_100K,

   /*!
   ** Baudrate 125 kBit/sec
   */
   CP_BAUD_125K,

   /*!
   ** Baudrate 250 kBit/sec
   */
   CP_BAUD_250K,

   /*!
   ** Baudrate 500 kBit/sec
   */
   CP_BAUD_500K,

   /*!
   ** Baudrate 800 kBit/sec
   */
   CP_BAUD_800K,

   /*!
   ** Baudrate 1 MBit/sec
   */
   CP_BAUD_1M,

   /*!
   ** Use values for Btr0 and Btr1
   */
   CP_BAUD_BTR,
   
   CP_BAUD_MAX = CP_BAUD_BTR
};


/*----------------------------------------------------------------------------*/
/*!
** \enum    CP_CHANNEL
** \brief   Channel definition
*/
enum CP_CHANNEL {
   CP_CHANNEL_1 = 0,
   CP_CHANNEL_2,
   CP_CHANNEL_3,
   CP_CHANNEL_4,
   CP_CHANNEL_5,
   CP_CHANNEL_6,
   CP_CHANNEL_7,
   CP_CHANNEL_8
};


/*----------------------------------------------------------------------------*/
/*!
** \enum    CP_MODE
** \brief   Mode of CAN controller
**
** These values are used as parameter for the function CpCoreCANMode() in
** order to change the state of the CAN controller.
*/
enum CP_MODE {
   /*!   Set controller in Stop mode (no reception / transmission possible)
   */
   CP_MODE_STOP = 0,

   /*!   Set controller into normal operation
   */
   CP_MODE_START,

   /*!	Start Autobaud Detection
   */
   CP_MODE_AUTO_BAUD,

   /*!	Set controller into Sleep mode
   */
   CP_MODE_SLEEP
};


/*----------------------------------------------------------------------------*/
/*!
** \enum    CP_STATE
** \brief   State of CAN controller
**
** These values are used as return value for the function CpCoreCANState().
*/
enum CP_STATE {
   /*!
   ** CAN controller is in stopped mode
   */
   CP_STATE_STOPPED  = 0,

   
   /*!
   **	CAN controller is active, no errors
   */
   CP_STATE_ACTIVE,


	/*!
	**	CAN controller is in Sleep mode
	*/
	CP_STATE_SLEEPING,

	/*!
	**	CAN controller is active, warning level is reached
	*/
	CP_STATE_BUS_WARN  = 6,

 	/*!
	**	CAN controller is error passive
	*/
	CP_STATE_BUS_PASSIVE,

	/*!
	**	CAN controller went into Bus Off
	*/
	CP_STATE_BUS_OFF,

	/*!
	**	General failure of physical layer detected (if supported by hardware)
	*/
	CP_STATE_PHY_FAULT = 10,

	/*!
	**	Fault on CAN-H detected (Low Speed CAN)
	*/
	CP_STATE_PHY_H,

	/*!
	**	Fault on CAN-L detected (Low Speed CAN)
	*/
	CP_STATE_PHY_L,

	CP_STATE_ERR_BIT   = 0x10,
	CP_STATE_ERR_STUFF = 0x20,
	CP_STATE_ERR_FORM	 = 0x30,
	CP_STATE_ERR_CRC   = 0x40,
	CP_STATE_ERR_ACK   = 0x50
};


/*----------------------------------------------------------------------------*/
/*!
** \enum    CP_BUFFER
** \brief   Buffer definition
**
** The enumeration CP_BUFFER is used to define a message buffer inside a
** FullCAN controller. The index for the first buffer starts at 1.
*/
enum CP_BUFFER {
   CP_BUFFER_1 = 1,
   CP_BUFFER_2,
   CP_BUFFER_3,
   CP_BUFFER_4,
   CP_BUFFER_5,
   CP_BUFFER_6,
   CP_BUFFER_7,
   CP_BUFFER_8,
   CP_BUFFER_9,
   CP_BUFFER_10,
   CP_BUFFER_11,
   CP_BUFFER_12,
   CP_BUFFER_13,
   CP_BUFFER_14,
   CP_BUFFER_15
};


/*----------------------------------------------------------------------------*/
/*!
** \enum    CP_BUFFER_DIR
** \brief   Buffer direction definition
*/
enum CP_BUFFER_DIR {
   /*!
   **    Buffer direction is receive
   */
   CP_BUFFER_DIR_RX = 0,

   /*!
   **    Buffer direction is transmit
   */
   CP_BUFFER_DIR_TX
};


/*----------------------------------------------------------------------------*\
** Structures                                                                 **
**                                                                            **
\*----------------------------------------------------------------------------*/


struct CpTime_s {
   _U32  ulSec1970;
   _U32  ulNanoSec;
};

typedef struct CpTime_s  _TsCpTime;
   

/*----------------------------------------------------------------------------*/
/*!
** \struct  CpCanMsg_s   canpie.h
** \brief   CAN message structure
**
** For transmission and reception of CAN messages a structure which holds
** all necessary informations is used. The structure has the following
** data fields:
*/
struct CpCanMsg_s {

   /*!   The identifier field may have 11 bits for standard frames
   **    (CAN specification 2.0A) or 29 bits for extended frames
   **    (CAN specification 2.0B). The three most significant bits
   **    are reserved (always read 0).
   */
   union {
      _U16  uwStd;
      _U32  ulExt;
   } tuMsgId;
   
   /*!   The data field has up to 8 bytes (64 bit) of message data.
   **    The number of used bytes is described via the structure
   **    member <b>ubDLC</b>.
   */
   union {
      _U08  aubByte[8];
      _U16  auwWord[4];
      _U32  aulLong[2];
   } tuMsgData;

   
   /*!   The data length code denotes the number of data bytes 
   **    which are transmitted by a message.
   **    The possible value range for the data length code is
   **    from 0 to 8 (bytes).<p>
   */
   _U08  ubMsgDLC;

   /*!   The structure member <b>ubMsgCtrl</b> defines the
   **    different data frames (2.0A / 2.0B) and the RTR frames.
   **    <ul>
   **    <li>Bit 0: Extended Frame if set to 1, else Standard Frame
   **    <li>Bit 1: Remote Frame if set to 1, else Data Frame
   **    </ul>   
   */
   _U08  ubMsgCtrl;


#if CP_CAN_MSG_TIME == 1
   /*!   The time stamp field defines the time when a CAN message
   **    was received by the CAN controller. The time stamp is a
   **    relative value, which is created by a free running timer.
   **    The time base is one microsecond (1 us). This means a
   **    maximum time span of 4294,96 seconds (1 hour 11 minutes)
   **    between two messages can be measured. This is an optional
   **    field (available if #CP_CAN_MSG_TIME is set to 1).
   */
   _TsCpTime ulMsgTime;
#endif

#if CP_CAN_MSG_USER == 1

   /*!   The field user data can hold a 32 bit value, which is
   **    defined by the user. This is an optional field
   **    (available if #CP_CAN_MSG_USER is set to 1).
   */
   _U32  ulMsgUser;
#endif

};



/*----------------------------------------------------------------------------*/
/*!
** \typedef _TsCpCanMsg
** \brief   CAN message structure
**
** For transmission and reception of CAN messages the structure CpCanMsg_s
** is used.
*/
typedef struct CpCanMsg_s  _TsCpCanMsg;


/*----------------------------------------------------------------------------*/
/*!
** \struct  CpStruct_HDI   canpie.h
** \brief   Hardware description interface
**
** The Hardware Description Interface provides a method to gather
** information about the CAN hardware and the functionality of the driver.
** All items in the structure CpStruct_HDI are constant and must be
** supplied by the designer of the CAN driver. The hardware description
** structure is available for every physical CAN channel.
*/

struct CpHdi_s {
   /*!   Bit coded value that decribes the features of the CAN driver.
   **    <ul>
   **    <li>Bit 0/1: 0 = Standard Frame, 1 = Extended Frame passive,
   **        2 = Extended Frame active
   **    <li>Bit 2: 0 = BasicCAN, 1 = FullCAN
   **    <li>Bit 3: 0 = No IRQ Handler, 1 = Has IRQ Handler
   **    <li>Bit 4: 0 = No identifier filter, 1 = software identifier filter
   **    <li>Bit 5: 0 = No timestamp, 1 = has timestamp
   **    <li>Bit 6: 0 = No user data field, 1 = has user data field
   **    <li>Bit 7: reserved
   **    </ul>
   */
   _U08  ubSupportFlags;

   /*!   Constant value that identifies the used CAN controller
   **    chip. Possible values for this member are listed
   **    in the header file cp_cc.h
   */
   _U08  ubControllerType;

   /*!   Defines the number of the interrupt which is used.
   **    If the flag IRQHandler is set to 0, the value of
   **    IRQNumber will be undefined.
   */
   _U08  ubIRQNumber;

   /*!   Holds the major version number of the CANpie driver
   **    release (current value = 0).
   */
   _U08  ubVersionMajor;

   /*!   Holds the minor version number of the CANpie
   **    driver release (current value = 4).
   */
   _U08  ubVersionMinor;

   /*!   A null terminated string which holds the name of the
   **    specific CAN driver. The string can be used to
   **    distinguish between different vendors.
   */
   char* szDriverName;

   /*!   A null terminated string which holds the version of
   **    the specific CAN driver (e.g. DLL version, firmware
   **    version).
   */
   char* szVersionNumber;
};


/*----------------------------------------------------------------------------*/
/*!
** \typedef CpStruct_HDI
** \brief   Hardware description interface structure
**
** The structure CpHdi_s provides fields to gather information about
** the CAN hardware. This typedef makes it compatible to older versions 
** of CANpie. Please use _TsCpHdi for new applications.
*/
typedef struct CpHdi_s  CpStruct_HDI;


/*----------------------------------------------------------------------------*/
/*!
** \typedef _TsCpHdi
** \brief   Hardware description interface structure
**
** The structure CpHdi_s provides fields to gather information about
** the CAN hardware.
*/
typedef struct CpHdi_s  _TsCpHdi;


/*----------------------------------------------------------------------------*/
/*!
** \struct  CpBitTiming_s   canpie.h
** \brief   Bit timing structure
** 
*/
struct CpBitTiming_s {
   
   /*!   holds the value from enum CP_BAUD
   */
   _U08     ubBaudSel;
   
   /*!   Bit timing register 0
   */
   _U08     ubBtr0;

   /*!   Bit timing register 1
   */
   _U08     ubBtr1;

   /*!   Syncronisation jump width
   */
   _U08     ubSjw;

   
   /*!   Baudrate prescaler
   */
   _U08     ubBrp;
};


typedef struct CpBitTiming_s  _TsCpBitrate;


/*----------------------------------------------------------------------------*/
/*!
** \struct  CpStats_s   canpie.h
** \brief   CAN statistic structure
** 
*/
struct CpStats_s {

   /*!   Total number of received data & remote frames
   */
   _U32     ulRcvMsgCount;

   /*!   Total number of transmitted data & remote frames
   */
   _U32     ulTrmMsgCount;

   /*!   Total number of error frames
   */
   _U32     ulErrMsgCount;
   
   /*!   Total number of errors, denoting a Bit error
   */
   _U32     ulErrMsg_Bit;

   _U32     ulErrMsg_Stuff;

   _U32     ulErrMsg_Form;

   _U32     ulErrMsg_Crc;

   _U32     ulErrMsg_Ack;

   _U08     ubRcvErrCounter;

   _U08     ubTrmErrCounter;

   _U08     ubBusLoad;
};

typedef struct CpStats_s _TsCpStats;



#endif   /* _CANPIE_H_   */

