//****************************************************************************//
// File:          canpie_dbg.h                                                //
// Description:                                                               //
// Author:        Uwe Koppe                                                   //
// e-mail:        koppe@microcontrol.net                                      //
//                                                                            //
//============================================================================//
// This program is free software; you can redistribute it and/or modify       //
// it under the terms of the GNU Lesser General Public License as published   //
// by the Free Software Foundation; either version 2.1 of the License, or     //
// (at your option) any later version.                                        //
//============================================================================//
//                                                                            //
// Date        History                                                        //
// ----------  -------------------------------------------------------------- //
// 29.07.2003  Initial version                                                //
//                                                                            //
//****************************************************************************//


#ifndef  _CANPIE_DBG_H_
#define  _CANPIE_DBG_H_


//------------------------------------------------------------------------------
// CVS version information:
// $Id: canpie_dbg.h,v 1.1 2005/06/01 21:18:19 microcontrol Exp $
//------------------------------------------------------------------------------


/*----------------------------------------------------------------------------*\
** Include files                                                              **
**                                                                            **
\*----------------------------------------------------------------------------*/
#include <linux/kernel.h>


//-----------------------------------------------------------------------------
/*!   \file    canpie_dbg.h
**    \brief   Debugging definitions for CANpie
**
**
*/

enum Debug_e {
   eDBG_NONE = 0,          // no debugging
   eDBG_PORT,              // debug port access
   eDBG_FUNC,              // debug CANpie function access
   eDBG_SOCK,              // debug socket access
   eDBG_DATA,              // debug CAN data
   eDBG_MAX = eDBG_DATA    // highest debug level
};

//---------------------------------------------------------
// PK_INF: print kernel information
//
#undef  PK_INF
#define PK_INF(fmt, args...)  printk(KERN_INFO  "canpie: " fmt "\n", ##args)


//---------------------------------------------------------
// PK_ERR: print kernel error
//
#undef  PK_ERR
#define PK_ERR(fmt, args...)  printk(KERN_ERR   "canpie: " fmt "\n", ##args)


//---------------------------------------------------------
// PK_DBG: print kernel debugging
//
#ifdef  CP_DEBUG
extern unsigned short uwCpDebugG;
#define PK_DBG(n, fmt, args...) if(n <= uwCpDebugG) \
   { printk(KERN_DEBUG "canpie: ==> %24s(): " fmt "\n", __FUNCTION__, ##args);}
#else
#define PK_DBG(n, fmt, args...)
#endif




#endif   /* _CANPIE_DBG_H_ */
