/*****************************************************************************\
*  CANpie                                                                     *
*                                                                             *
*  File        : cp_arch.h                                                    *
*  Description : Definitions / Constants for CANpie                           *
*  Author      : Uwe Koppe                                                    *
*  e-mail      : koppe@microcontrol.net                                       *
*                                                                             *
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
*                                                                             *
*   This program is free software; you can redistribute it and/or modify      *
*   it under the terms of the GNU General Public License as published by      *
*   the Free Software Foundation; either version 2 of the License, or         *
*   (at your option) any later version.                                       *
*                                                                             *
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
*                                                                             *
*  History                                                                    *
*  Vers.    Date        Comment                                         Aut.  *
*  -----    ----------  ---------------------------------------------   ----  *
*                                                                             *
\*****************************************************************************/


#ifndef  _CP_ARCH_H_
#define  _CP_ARCH_H_

//------------------------------------------------------------------------------
// CVS version information:
// $Id: cp_arch.h,v 1.3 2005/08/15 16:29:40 microcontrol Exp $
//------------------------------------------------------------------------------


//-----------------------------------------------------------------------------
/*!   \file    cp_arch.h
**    \brief   CANpie constants, structures and enumerations
**
**    This file does the setup for various targets
**
*/


/*----------------------------------------------------------------------------*\
** Include files                                                              **
**                                                                            **
\*----------------------------------------------------------------------------*/
#include "compiler.h"      // compiler definitions
#include "cp_cc.h"         // definitions for CAN controller / target


/*----------------------------------------------------------------------------*\
** Definitions & Enumerations                                                 **
**                                                                            **
\*----------------------------------------------------------------------------*/

//-------------------------------------------------------------------
// The symbol CP_TARGET defines the target for the CANpie sources.
// 
#ifndef CP_TARGET
#error  Target (Symbol CP_TARGET) is not defined! Check file cp_arch.h!
#endif


/*----------------------------------------------------------------------------*\
** Structures                                                                 **
**                                                                            **
\*----------------------------------------------------------------------------*/




/*----------------------------------------------------------------------------*/
/*!
** \struct  CpPortLinux_s   cp_arch.h
** \brief   Port structure for Linux
** 
*/
struct CpPortLinux_s {

   /*!   logical CAN interface number, 
   **    first index is 0, value -1 denotes not assigned
   */
   int      slLogIf;

   /*!   physical CAN interface number, 
   **    first index is 0, value -1 denotes not assigned
   */
   int      slPhyIf;
   
   /*!   CAN message queue number, 
   **    first index is 0, value -1 denotes not assigned
   */
   int      slQueue;
};



//---------------------------------------------------------------------
// Architecture definitions for Linux
//
#if CP_TARGET == CP_CC_LINUX

typedef struct CpPortLinux_s     _TsCpPort;

typedef _U08                     _TvCpStatus;
#else
#error CP_TARGET is wrong
#endif



#endif   /* _CP_ARCH_H_   */

