//****************************************************************************//
// Project     :  CANpie                                                      //
// File        :  cp_cc.h                                                     //
// Description :  Enumerations for CAN controller chips / targets             //
// Author:        Uwe Koppe                                                   //
// e-mail:        koppe@microcontrol.net                                      //
//                                                                            //
//============================================================================//
// This program is free software; you can redistribute it and/or modify       //
// it under the terms of the GNU Lesser General Public License as published   //
// by the Free Software Foundation; either version 2.1 of the License, or     //
// (at your option) any later version.                                        //
//============================================================================//
//                                                                            //
// Date        History                                                        //
// ----------  -------------------------------------------------------------- //
// 29.07.2003  Initial version                                                //
//                                                                            //
//****************************************************************************//


#ifndef  _CP_CC_H_
#define  _CP_CC_H_

/*-----------------------------------------------------------------------------
**    Online documentation for Doxygen
*/

/*!   \file    canpie.h
**    \brief   CANpie constants, structures and enumerations
**
**    This file holds constants and structures used within CANpie.
**
*/



/*----------------------------------------------------------------------------*/
/*!
** \enum    CP_CC
** \brief   CAN controller identification numbers
**
*/
enum CP_CC {

   //-------------------------------------------------------//
   // Generic                                               //
   //-------------------------------------------------------//
   //!   LINUX
   CP_CC_LINUX = 0x0010,

   //-------------------------------------------------------//
   // Analog Devices    (0x08xx)                            //
   //-------------------------------------------------------//

   //-------------------------------------------------------//
   // ATMEL             (0x10xx)                            //
   //-------------------------------------------------------//

   //-------------------------------------------------------//
   // Bosch             (0x18xx)                            //
   //-------------------------------------------------------//

   //-------------------------------------------------------//
   // Cygnal            (0x20xx)                            //
   //-------------------------------------------------------//
   
   //-------------------------------------------------------//
   // Dallas            (0x28xx)                            //
   //-------------------------------------------------------//
   
   //-------------------------------------------------------//
   // Fujitsu           (0x30xx)                            //
   //-------------------------------------------------------//
   
   //-------------------------------------------------------//
   // Infineon          (0x38xx)                            //
   //-------------------------------------------------------//
   //!   Infineon C505
   CP_CC_C505 = 0x3800,

   //!   Infineon C515
   CP_CC_C515,

   //!   Infineon C161
   CP_CC_C161,

   //!   Infineon C164
   CP_CC_C164,

   //!   Infineon C167
   CP_CC_C167,

   //!   Infineon 81C90
   CP_CC_81C90,

   //!   Infineon 81C91
   CP_CC_81C91,

   //-------------------------------------------------------//
   // Microchip         (0x40xx)                            //
   //-------------------------------------------------------//
   
   //-------------------------------------------------------//
   // Micronas          (0x48xx)                            //
   //-------------------------------------------------------//
   
   //-------------------------------------------------------//
   // Motorola          (0x50xx)                            //
   //-------------------------------------------------------//
      
   //-------------------------------------------------------//
   // National Semi.    (0x58xx)                            //
   //-------------------------------------------------------//
   
   //-------------------------------------------------------//
   // NEC               (0x60xx)                            //
   //-------------------------------------------------------//

   //-------------------------------------------------------//
   // Philips           (0x68xx)                            //
   //-------------------------------------------------------//
   //!   Philips 82C200                                     
   CP_CC_82C200 = 0x6800,

   //!   Philips SJA1000
   CP_CC_SJA1000,

   //!   Philips 80C591
   CP_CC_80C591,

   //!   Philips 80C592
   CP_CC_80C592,
   
   //-------------------------------------------------------//
   // Renesas           (0x70xx)                            //
   //-------------------------------------------------------//
   
   //-------------------------------------------------------//
   // ST                (0x78xx)                            //
   //-------------------------------------------------------//

   //-------------------------------------------------------//
   // Texas Instruments (0x80xx)                            //
   //-------------------------------------------------------//
   
   //-------------------------------------------------------//
   // Toshiba           (0x88xx)                            //
   //-------------------------------------------------------//


   //-------------------------------------------------------//
   // Last entry                                            //
   //-------------------------------------------------------//
   CP_CC_UNDEF = 0xFF00

};




#endif   /* _CP_CC_H_   */

