//****************************************************************************//
// File:          cp_loop.c                                                   //
// Description:   CANpie core module for Linux                                //
// Author:        Uwe Koppe                                                   //
// e-mail:        koppe@microcontrol.net                                      //
//                                                                            //
//============================================================================//
// This program is free software; you can redistribute it and/or modify       //
// it under the terms of the GNU Lesser General Public License as published   //
// by the Free Software Foundation; either version 2.1 of the License, or     //
// (at your option) any later version.                                        //
//============================================================================//
//                                                                            //
// Date        History                                                        //
// ----------  -------------------------------------------------------------- //
// 02.01.2004  Initial version                                                //
//                                                                            //
//****************************************************************************//


//------------------------------------------------------------------------------
// CVS version information:
// $Id: can_loop.c,v 1.1 2005/06/01 21:47:17 microcontrol Exp $
//------------------------------------------------------------------------------


/*----------------------------------------------------------------------------*\
** Include files                                                              **
**                                                                            **
\*----------------------------------------------------------------------------*/

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>

#include <net/can/canpie.h>
#include <net/can/canpie_hal.h>
#include <net/can/cpcanmsg.h>


/*----------------------------------------------------------------------------*\
** Definitions                                                                **
**                                                                            **
\*----------------------------------------------------------------------------*/

//---------------------------------------------------------
// PK_DBG: print kernel debug stuff
//
#ifdef  CP_DEBUG
#undef  PK_DBG
#define PK_DBG(fmt, args...)  printk(KERN_DEBUG "cp_loop: => %24s(): " fmt "\n", __FUNCTION__, ##args);
#else
#define PK_DBG(fmt, args...)  
#endif

//---------------------------------------------------------
// PK_INF: print kernel information
//
#undef  PK_INF
#define PK_INF(fmt, args...)  printk(KERN_INFO  "cp_loop: " fmt "\n", ##args)


//---------------------------------------------------------
// PK_ERR: print kernel error
//
#undef  PK_ERR
#define PK_ERR(fmt, args...)  printk(KERN_ERR   "cp_loop: " fmt "\n", ##args)



#define CP_LOOP_VERSION      "1.0"

#define MAX_MAILBOX           14        // number of virtual mailboxes

#define MAX_PHY_IF            4        // maximum number of "physical" interfaces


/*----------------------------------------------------------------------------*\
** Module information                                                         **
**                                                                            **
\*----------------------------------------------------------------------------*/
MODULE_AUTHOR("Uwe Koppe <koppe@microcontrol.net");
MODULE_DESCRIPTION("CANpie loopback module");
MODULE_LICENSE("GPL");


/*----------------------------------------------------------------------------*\
** Module variables                                                           **
**                                                                            **
\*----------------------------------------------------------------------------*/

static int           port = -1;
MODULE_PARM(port, "i");
MODULE_PARM_DESC(port, "Logic port number ");



static char * szDriverNameS = "cp_loop";
static char * szDriverVersS = "1.00";

static _TsCpCanMsg   atsMsgBufferS[MAX_MAILBOX];


static int           aslCanLogIf[MAX_PHY_IF];


/*----------------------------------------------------------------------------*\
** Functions                                                                  **
**                                                                            **
\*----------------------------------------------------------------------------*/



//----------------------------------------------------------------------------//
// CpLoopBitrate()                                                            //
// Setup baudrate of CAN controller                                           //
//----------------------------------------------------------------------------//
_TvCpStatus CpLoopBitrate(_TsCpPort * ptsPortV, _TsCpBitrate * ptsBitrateV)
{
   _U08  ubReturnT = CpErr_OK;
   
   //----------------------------------------------------------------
   // test the channel number
   //


   return(ubReturnT);
}


//----------------------------------------------------------------------------//
// CpLoopBufferGetDlc()                                                       //
// Get DLC of buffer                                                          //
//----------------------------------------------------------------------------//
_U08 CpLoopBufferGetDlc(_TsCpPort * ptvPortV, _U08 ubBufferV, _U08 * pubDlcV)
{
   _TsCpCanMsg *   ptsCanMsgT;
     
   //----------------------------------------------------------------
   // test the channel number
   //

   
   //----------------------------------------------------------------
   // test the buffer number
   //
   if(ubBufferV < CP_BUFFER_1) return (CpErr_BUFFER);
   if(ubBufferV > MAX_MAILBOX) return (CpErr_BUFFER);
   
   ptsCanMsgT = &atsMsgBufferS[ubBufferV - 1];
   *pubDlcV = CpMsgGetDlc(ptsCanMsgT);
   
   return(CpErr_OK);
}


//----------------------------------------------------------------------------//
// CpLoopBufferInit()                                                         //
// Initialize buffer of FullCAN controller                                    //
//----------------------------------------------------------------------------//
_U08 CpLoopBufferInit(_TsCpPort * ptvPortV, _TsCpCanMsg * ptsCanMsgV,
                      _U08 ubDirectionV)
{
   //----------------------------------------------------------------
   // test the channel number
   //

   
   return (CpErr_OK);
}


//----------------------------------------------------------------------------//
// CpLoopBufferRelease()                                                      //
// release a FullCAN buffer                                                   //
//----------------------------------------------------------------------------//
_U08 CpLoopBufferRelease(_TsCpPort * ptvPortV, _U08 ubBufferV)
{
   //----------------------------------------------------------------
   // test the channel number
   //
   
   
   //----------------------------------------------------------------
   // test the buffer number
   //
   if(ubBufferV < CP_BUFFER_1) return (CpErr_BUFFER);
   if(ubBufferV > MAX_MAILBOX) return (CpErr_BUFFER);

   return (CpErr_OK);
}


//----------------------------------------------------------------------------//
// CpLoopBufferSetDlc()                                                       //
// Set DLC of buffer                                                          //
//----------------------------------------------------------------------------//
_U08 CpLoopBufferSetDlc(_TsCpPort * ptvPortV, _U08 ubBufferV, _U08 ubDlcV)
{
   //----------------------------------------------------------------
   // test the channel number
   //

   //----------------------------------------------------------------
   // test the buffer number
   //
   if(ubBufferV < CP_BUFFER_1) return (CpErr_BUFFER);
   if(ubBufferV > MAX_MAILBOX) return (CpErr_BUFFER);


   return (CpErr_OK);
}




//----------------------------------------------------------------------------//
// CpLoopBufferSend()                                                     		//
// Send message of FullCAN buffer                                      			//
//----------------------------------------------------------------------------//
_U08 CpLoopBufferSend(_U08 ubChannelV, _U08 ubBufferV)
{
   //----------------------------------------------------------------
   // test the channel number
   //
   if( ubChannelV >= CP_CHANNEL_MAX) return (CpErr_CHANNEL);

      
   //----------------------------------------------------------------
   // test the buffer number
   //
   if(ubBufferV < CP_BUFFER_1) return (CpErr_BUFFER);
   if(ubBufferV > MAX_MAILBOX) return (CpErr_BUFFER);

   return (CpErr_OK);
}


//----------------------------------------------------------------------------//
// CpLoopBufferTransmit()                                                     //
// Transmit contents of buffer                                                //
//----------------------------------------------------------------------------//
_U08 CpLoopBufferTransmit(_U08 ubChannelV, _TsCpCanMsg * ptsCanMsgV)
{
   //----------------------------------------------------------------
   // test the channel number
   //
   if( ubChannelV >= CP_CHANNEL_MAX) return (CpErr_CHANNEL);

   return (CpErr_OK);
}


//----------------------------------------------------------------------------//
// CpLoopCanMode()                                                            //
// Set mode of CAN controller                                                 //
//----------------------------------------------------------------------------//
_TvCpStatus CpLoopCanMode(_TsCpPort * ptsPortV, _U08 ubModeV)
{
   _U08  ubReturnT = CpErr_OK;

   //----------------------------------------------------------------
   // test the channel number
   //

   return(ubReturnT);
}


//----------------------------------------------------------------------------//
// CpLoopCanStatus()                                                          //
// Retrive status of CAN controller                                           //
//----------------------------------------------------------------------------//
_TvCpStatus CpLoopCanStatus(_TsCpPort * ptsPortV, _U08 * pubStatusV)
{
   _U08  ubStatusT = 0;
   _U08  ubReturnT = CpErr_OK;

   //----------------------------------------------------------------
   // test the channel number
   //



   *pubStatusV = ubStatusT;
           
   return(ubReturnT);
}


//----------------------------------------------------------------------------//
// CpLoopDriverInit()                                                         //
// Initialize the CAN driver                                                  //
//----------------------------------------------------------------------------//
_TvCpStatus CpLoopDriverInit(_U08 ubPhysPortV, _TsCpPort * ptvPortV)
{
   _U08  ubReturnT = CpErr_OK;



   //----------------------------------------------------------------
   // test the channel number
   //
   if( ubPhysPortV >= CP_CHANNEL_MAX) return (CpErr_CHANNEL);




   return(ubReturnT);
}


//----------------------------------------------------------------------------//
// CpLoopDriverRelease()                                                      //
// De-Initialize the CAN driver                                               //
//----------------------------------------------------------------------------//
_TvCpStatus CpLoopDriverRelease(_TsCpPort * ptvPortV)
{
   _U08  ubReturnT = CpErr_OK;




   return(ubReturnT);
}



//----------------------------------------------------------------------------//
// CpLoopHDI()                                                                //
// Fill Hardware Description Interface (HDI) structure                        //
//----------------------------------------------------------------------------//
_U08 CpLoopHDI(_U08 ubChannelV, _TsCpHdi * pHdiV)

{
   //----------------------------------------------------------------
   // test the channel number
   //
   if( ubChannelV >= CP_CHANNEL_MAX) return (CpErr_CHANNEL);

   if(pHdiV == 0L) return(CpErr_GENERIC);
   
   pHdiV->ubSupportFlags   = 0;
   pHdiV->ubControllerType = 0;
   pHdiV->ubIRQNumber      = 0;
   pHdiV->szDriverName     = szDriverNameS;
   pHdiV->szVersionNumber  = szDriverVersS;
   
   return (CpErr_OK);
}


//----------------------------------------------------------------------------//
// CpLoopFilterAll()                                                          //
// Enable / Disable filter for all messages                                   //
//----------------------------------------------------------------------------//
_U08 CpLoopFilterAll(_U08 ubChannelV, _BIT btEnableV)
{
   //----------------------------------------------------------------
   // test the channel number
   //
   if( ubChannelV >= CP_CHANNEL_MAX) return (CpErr_CHANNEL);


   return (CpErr_OK);
}


//----------------------------------------------------------------------------//
// CpLoopFilterMsg()                                                          //
// Enable / Disable filter for a single messages                              //
//----------------------------------------------------------------------------//
_U08 CpLoopFilterMsg(_U08 ubChannelV, _U16 uwIdV, _BIT btEnableV)
{
   //----------------------------------------------------------------
   // test the channel number
   //
   if( ubChannelV >= CP_CHANNEL_MAX) return (CpErr_CHANNEL);


   return (CpErr_OK);
}


//----------------------------------------------------------------------------//
// CpLoopMsgRead()                                                            //
// Read CAN message from controller                                           //
//----------------------------------------------------------------------------//
_TvCpStatus CpLoopMsgRead( _TsCpPort * ptsPortV, _TsCpCanMsg * ptsCanMsgV,
                           _U16 * puwMsgCntV)
{
   _U08  ubReturnT = CpErr_OK;
   _U16  uwMsgCntT = 0;
   

   //----------------------------------------------------------------
   // read data from the receive queue
   //
   uwMsgCntT = can_queue_rcv_pop(ptsPortV, ptsCanMsgV, *puwMsgCntV);
   *puwMsgCntV = uwMsgCntT;

   return(ubReturnT);
}


//----------------------------------------------------------------------------//
// CpLoopMsgTransmit()                                                        //
// Write CAN message to controller                                            //
//----------------------------------------------------------------------------//
_TvCpStatus CpLoopMsgWrite(_TsCpPort * ptsPortV, _TsCpCanMsg * ptsCanMsgV,
                           _U16 * puwMsgCntV)
{
   _U08        ubReturnT = CpErr_OK;
   _TsCpCanMsg tsCanMsgT;

   PK_DBG(  "copy data to log. I/F=%d , phy. I/F=%d", 
            ptsPortV->slLogIf, ptsPortV->slPhyIf);
            
   //----------------------------------------------------------------
   // copy data from the transmit queue into the receive queue
   //
   can_queue_trm_pop(ptsPortV, &tsCanMsgT, 1);
   
   PK_DBG(  "Message: ID=%04X DLC=%1d Data=%02X %02X", 
            tsCanMsgT.tuMsgId.ulExt, tsCanMsgT.ubMsgDLC, 
            tsCanMsgT.tuMsgData.aubByte[0],
            tsCanMsgT.tuMsgData.aubByte[1]);
   
   can_queue_rcv_push(ptsPortV, &tsCanMsgT, 1);

   return(ubReturnT);
}


//----------------------------------------------------------------------------//
// CpLoopRegRead()                                                            //
// Read one byte from specified CAN controller register                       //
//----------------------------------------------------------------------------//
_U08 CpLoopRegRead(_U08 ubChannelV, _U16 uwRegAdrV, _U08 * pubValueV)
{
   //----------------------------------------------------------------
   // test the channel number
   //
   if( ubChannelV >= CP_CHANNEL_MAX) return (CpErr_CHANNEL);


   return (CpErr_OK);
}


//----------------------------------------------------------------------------//
// CpLoopRegWrite()                                                           //
// Write one byte to specified CAN controller register                        //
//----------------------------------------------------------------------------//
_U08 CpLoopRegWrite(_U08 ubChannelV, _U16 uwRegAdrV, _U08 ubValueV)
{
   //----------------------------------------------------------------
   // test the channel number
   //
   if( ubChannelV >= CP_CHANNEL_MAX) return (CpErr_CHANNEL);


   return (CpErr_OK);
}


/*----------------------------------------------------------------------------*\
** module init and exit code                                                  **
** function implementation                                                    **
\*----------------------------------------------------------------------------*/


//----------------------------------------------------------------
// Setup the CpHal_s structure
//
static _TsCpHal tsMcLoopHalS = {
   pfnBitrate        : CpLoopBitrate,
   pfnBufferGetData  : 0L,
   pfnBufferGetDlc   : 0L,
   pfnBufferInit     : 0L,
   pfnBufferSetData  : 0L,
   pfnBufferSetDlc   : 0L,
   pfnBufferSend     : 0L,
   pfnBufferRelease  : 0L,
   pfnCanMode        : CpLoopCanMode,
   pfnCanStatus      : CpLoopCanStatus,
   pfnDriverInit     : CpLoopDriverInit,
   pfnDriverRelease  : CpLoopDriverRelease,
   pfnFilterAll      : 0L,
   pfnFilterMsg      : 0L,
   pfnHwDescription  : CpLoopHDI,

   pfnMsgRead        : CpLoopMsgRead,
   pfnMsgWrite       : CpLoopMsgWrite,
   pfnRegRead        : 0L,
   pfnRegWrite       : 0L

};





//----------------------------------------------------------------------------//
// init_module()                                                              //
//                                                                            //
//----------------------------------------------------------------------------//
static int __init CpLoopInit(void)
{
   int   slRegisterT;
   int   slCanIfIdxT;
   
      
   //----------------------------------------------------------------
   // print some info who we are
   //       
   PK_INF("CANpie loop driver version %s", CP_MODULE_VERSION);

   
   //----------------------------------------------------------------
   // clear the physical-to-logical CAN IF table
   //       
   for(slCanIfIdxT = 0; slCanIfIdxT < MAX_PHY_IF; slCanIfIdxT++)
   {
      aslCanLogIf[slCanIfIdxT] = -1;
   }
   
   slRegisterT = can_register_driver(port, 0, &tsMcLoopHalS);
   if(slRegisterT < 0)
   {
      return(-ENODEV);
   }
   
   aslCanLogIf[0] = slRegisterT;
   
   port = slRegisterT;
   PK_INF("Logic port number: %d", port);
   return(0);
}


//----------------------------------------------------------------------------//
// cleanup_module()                                                           //
//                                                                            //
//----------------------------------------------------------------------------//
static void __exit CpLoopExit(void)
{
   can_unregister_driver(0);
   PK_INF("CANpie Kernel driver removed");
}


module_init(CpLoopInit);
module_exit(CpLoopExit);
