//****************************************************************************//
// File:          mc_pcmcia_cs.c                                              //
// Description:   CANpie hardware abstraction layer (HAL)                     //
// Author:        Uwe Koppe                                                   //
// e-mail:        koppe@microcontrol.net                                      //
//                                                                            //
//============================================================================//
// This program is free software; you can redistribute it and/or modify       //
// it under the terms of the GNU Lesser General Public License as published   //
// by the Free Software Foundation; either version 2.1 of the License, or     //
// (at your option) any later version.                                        //
//============================================================================//
//                                                                            //
// Date        History                                                        //
// ----------  -------------------------------------------------------------- //
// 29.07.2003  Initial version                                                //
//                                                                            //
//****************************************************************************//


//------------------------------------------------------------------------------
// CVS version information:
// $Id: mc_pcmcia_cs.c,v 1.2 2005/06/01 22:16:55 microcontrol Exp $
//------------------------------------------------------------------------------




/*----------------------------------------------------------------------------*\
** Include files                                                              **
**                                                                            **
\*----------------------------------------------------------------------------*/

#include <linux/kernel.h>
#include <linux/module.h>           // must be included in every kernel module
#include <linux/init.h>
#include <linux/ioport.h>           // request_region() / release_region()
#include <linux/interrupt.h>        // request_irq() / free_irq()

#include <linux/sched.h>
#include <linux/ptrace.h>
#include <linux/slab.h>
#include <linux/string.h>
#include <linux/timer.h>
#include <asm/io.h>
#include <asm/system.h>


#include "mc_pcmcia_cs.h"


/*====================================================================*/

/* Parameters that can be set with 'insmod' */




MODULE_AUTHOR("Uwe Koppe <koppe@microcontrol.net>");
MODULE_DESCRIPTION("Driver for MicroControl PCMCIA CAN interface");
MODULE_LICENSE("GPL");


static unsigned short   auwMcPcmciaPortS[2] = {0, 1};
static int              slMcPcmciaPortCntS = 2;
module_param_array_named(port, auwMcPcmciaPortS, ushort, &slMcPcmciaPortCntS, 0);
MODULE_PARM_DESC(port, " Port number");


/*----------------------------------------------------------------------------*\
** Debugging                                                                  **
**                                                                            **
\*----------------------------------------------------------------------------*/
#ifdef  CP_DEBUG
unsigned short    uwMcPcmciaDebugG = 2;
module_param_named(debug, uwMcPcmciaDebugG, ushort, 0);
MODULE_PARM_DESC(debug, " Set debug level, 0=quiet .. 5=noisy");
#endif


/*====================================================================*/

/*
   The event() function is this driver's Card Services event handler.
   It will be called by Card Services when an appropriate card status
   event is received.  The config() and release() entry points are
   used to configure or release a socket, in response to card
   insertion and ejection events.  They are invoked from the dummy
   event handler. 
*/

static void mc_pcmcia_config(dev_link_t *link);
static void mc_pcmcia_detach(dev_link_t *link);
static int  mc_pcmcia_event(event_t event, int priority, event_callback_args_t *args);
static void mc_pcmcia_release(dev_link_t *link);



//-------------------------------------------------------------------
// The dev_info variable is the "key" that is used to match up this
// device driver with appropriate cards, through the card 
// configuration database.
//
static dev_info_t dev_info = CP_DRV_NAME;

//-------------------------------------------------------------------
// A linked list of "instances" of the dummy device.  Each actual
// PCMCIA card corresponds to one device instance, and is described
// by one dev_link_t structure (defined in ds.h).
//
static dev_link_t *dev_list = NULL;

 


//----------------------------------------------------------------------------//
// cs_error()                                                                 //
// Handle errors of the Card Service                                          //
//----------------------------------------------------------------------------//
/*
static void cs_error(client_handle_t handle, int func, int ret)
{
    error_info_t err = { func, ret };
    // CardServices(ReportError, handle, &err);
}
*/

//----------------------------------------------------------------------------//
// canpie_attach()                                                            //
// This function creates an "instance" of the driver, allocating local data   //
// structures for one device.  The device is registered with Card Services.   //
// The dev_link structure is initialized, but we don't actually configure the //
// card at this point -- we wait until we receive a card insertion event.     //
//----------------------------------------------------------------------------//
static dev_link_t * mc_pcmcia_attach(void)
{
   _TsMcPcmciaInfo * info;
   dev_link_t *      link;
   client_reg_t      client_reg;
   int               ret;
    
   PK_DBG("mc_pcmcia_attach()");

   //-------------------------------------------
   // Create new PCMCIA CAN device
   //
   info = kmalloc(sizeof(*info), GFP_KERNEL);
   if (!info) return NULL;
   memset(info, 0, sizeof(*info));
   link = &info->link; link->priv = info;

      
   link->io.Attributes1 = IO_DATA_PATH_WIDTH_8;
   link->io.Attributes2 = IO_DATA_PATH_WIDTH_8;
   link->irq.Attributes = IRQ_TYPE_EXCLUSIVE;
   link->irq.IRQInfo1   = 0;  // No IRQ supported for this card
    
   link->conf.Attributes = 0;
   link->conf.Vcc = 50;
   link->conf.IntType = INT_MEMORY_AND_IO;
    
   //----------------------------------------------------------------
   // Register with Card Services 
   //
   link->next = dev_list;
   dev_list = link;
   client_reg.dev_info = &dev_info;
   client_reg.Attributes = INFO_IO_CLIENT | INFO_CARD_SHARE;
   client_reg.EventMask  = CS_EVENT_CARD_INSERTION | CS_EVENT_CARD_REMOVAL |
                           CS_EVENT_RESET_PHYSICAL | CS_EVENT_CARD_RESET |
                           CS_EVENT_PM_SUSPEND | CS_EVENT_PM_RESUME;
                           
   client_reg.event_handler = &mc_pcmcia_event;
   client_reg.Version = 0x0210;
   client_reg.event_callback_args.client_data = link;
   ret = pcmcia_register_client(&link->handle, &client_reg);
   if (ret != CS_SUCCESS) 
   {
      cs_error(link->handle, RegisterClient, ret);
      mc_pcmcia_detach(link);
      return NULL;
   }
    
   return link;
}


//----------------------------------------------------------------------------//
// mc_pcmcia_detach()                                                         //
// This deletes a driver "instance".  The device is de-registered with Card   //
// Services.  If it has been released, all local data structures are freed.   //
// Otherwise, the structures will be freed when the device is released.       //
//                                                                            //
//----------------------------------------------------------------------------//
static void mc_pcmcia_detach(dev_link_t *link)
{
	dev_link_t **linkp;

   PK_DBG("mc_pcmcia_detach()");

   if(link == NULL) return;

   //----------------------------------------------------------------
   // Locate device structure
   //
   for(linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
   {
      if(*linkp == link) break;
      if(*linkp == NULL) return;
   }


   //----------------------------------------------------------------
   // If the device is currently configured and active, we won't
   // actually delete it yet.  Instead, it is marked so that when
   // the release() function is called, that will trigger a proper
   //  detach().
   //
   if(link->state & DEV_CONFIG)
   {
      #ifdef PCMCIA_DEBUG
      printk(KERN_DEBUG "canpie_cs() detach postponed, '%s' still locked\n",
             link->dev->dev_name);
      #endif

      link->state |= DEV_STALE_LINK;
      return;
   }

   //----------------------------------------------------------------
   // Break the link with Card Services
   //
   if(link->handle)
   {
      pcmcia_deregister_client(link->handle);
    
      //--- Unlink device structure, and free it ---------------
      *linkp = link->next;

      //--- This points to the mc_pcmcia_info_t struct ------
      kfree(link->priv);
   }
}



#define CS_CHECK(fn, ret) \
do { last_fn = (fn); if ((last_ret = (ret)) != 0) goto cs_failed; } while (0)

//----------------------------------------------------------------------------//
// mc_pcmcia_config()                                                         //
// This function is scheduled to run after a CARD_INSERTION event is          //
// received, to configure the PCMCIA socket, and to make the device           //
// available to the system.                                                   //
//                                                                            //
//----------------------------------------------------------------------------//
void mc_pcmcia_config(dev_link_t *link)
{
   client_handle_t         handle = link->handle;
   _TsMcPcmciaInfo *       dev = link->priv;
   tuple_t                 tuple;
   u_short                 buf[128];
   cisparse_t              parse;
   config_info_t           conf;
   cistpl_cftable_entry_t *cfg = &parse.cftable_entry;
   cistpl_cftable_entry_t  dflt = { 0 };
   int last_ret, last_fn;

       
   PK_DBG("mc_pcmcia_config");
    
   tuple.TupleData      = (cisdata_t *)buf;
   tuple.TupleOffset    = 0; 
   tuple.TupleDataMax   = 255;
   tuple.Attributes     = 0;
   tuple.DesiredTuple   = CISTPL_CONFIG;
   
   CS_CHECK(GetFirstTuple, pcmcia_get_first_tuple(handle, &tuple));
   CS_CHECK(GetTupleData,  pcmcia_get_tuple_data(handle, &tuple));
   CS_CHECK(ParseTuple,    pcmcia_parse_tuple(handle, &tuple, &parse));
   link->conf.ConfigBase = parse.config.base;
   link->conf.Present    = parse.config.rmask[0];
    
   //----------------------------------------------------------------
   // Configure card
   //
   link->state |= DEV_CONFIG;

   //----------------------------------------------------------------
   // Lookup the current Vcc
   //
   CS_CHECK(GetConfigurationInfo, pcmcia_get_configuration_info(handle, &conf));
   link->conf.Vcc = conf.Vcc;
   

   //----------------------------------------------------------------
   // In this loop, we scan the CIS for configuration table entries,
   // each of which describes a valid card configuration, including
   // voltage, IO window, memory window, and interrupt settings.
   //
   tuple.DesiredTuple   = CISTPL_CFTABLE_ENTRY;
   tuple.Attributes     = 0;
   CS_CHECK(GetFirstTuple, pcmcia_get_first_tuple(handle, &tuple));
   while (1) 
   {
      
      if( pcmcia_get_tuple_data(handle, &tuple) != 0 ||
          pcmcia_parse_tuple(handle, &tuple, &parse) != 0)
      {
         goto next_entry;
      }
      
      //--------------------------------------------------------
      // IRQ settings
      //
      /*
      if(cfg->irq.IRQInfo1 || dflt.irq.IRQInfo1)
      {
         link->conf.Attributes |= CONF_ENABLE_IRQ;
      }
      */
      
      //--------------------------------------------------------
      // I/O window settings
      //
      link->io.NumPorts1 = 0;
      link->io.NumPorts2 = 0;
      if( (cfg->io.nwin > 0) || (dflt.io.nwin > 0) ) 
      {
         cistpl_io_t *io = (cfg->io.nwin) ? &cfg->io : &dflt.io;
         link->io.Attributes1 = IO_DATA_PATH_WIDTH_AUTO;
         if( !(io->flags & CISTPL_IO_8BIT) )
         {
            link->io.Attributes1 = IO_DATA_PATH_WIDTH_16;
         }
         if( !(io->flags & CISTPL_IO_16BIT))
         {
            link->io.Attributes1 = IO_DATA_PATH_WIDTH_8;
         }
         
         link->io.IOAddrLines = io->flags & CISTPL_IO_LINES_MASK;
         link->io.BasePort1   = io->win[0].base;
         link->io.NumPorts1   = io->win[0].len;
         if(io->nwin > 1) 
         {
            link->io.Attributes2 = link->io.Attributes1;
            link->io.BasePort2   = io->win[1].base;
            link->io.NumPorts2   = io->win[1].len;
         }
         
         //------------------------------------------------
         // This reserves IO space but doesn't actually 
         // enable it
         //
         if(pcmcia_request_io(link->handle, &link->io) != 0)
         {
            goto next_entry;
         }
         
      }
    
      //-----------------------------------------------
      // If we've got this far, we're done
      //
      break;
   
      next_entry:
      if(cfg->flags & CISTPL_CFTABLE_DEFAULT) dflt = *cfg;
      CS_CHECK(GetNextTuple, pcmcia_get_next_tuple(handle, &tuple));
   }

   // link->irq.Attributes = IRQ_FORCED_PULSE;
   //CS_CHECK(RequestIRQ, pcmcia_request_irq(handle, &link->irq));
   PK_DBG("Conf. I/O-Base=%04X I/O-Num=%02d  IRQ=%02d Attributes=%X", 
            link->io.BasePort1, link->io.NumPorts1, 
            link->irq.AssignedIRQ, link->irq.Attributes);

   request_region(link->io.BasePort1, link->io.NumPorts1, CP_DRV_NAME);
   
   CS_CHECK(RequestConfiguration, pcmcia_request_configuration(handle, &link->conf));
   
   
   sprintf(dev->node.dev_name, CP_DRV_NAME);
   dev->node.major = 0;
   dev->node.minor = 0;
   dev->irg = 0;
   dev->port = link->io.BasePort1;
   link->dev = &dev->node;

   //----------------------------------------------------------------
   // reset the PCMCIA card
   //
   mc_pcmcia_set_info(link->priv);
   mc_pcmcia_reset();
   mc_pcmcia_task_start();

   link->state &= ~DEV_CONFIG_PENDING;
   return;
    
cs_failed:
   cs_error(link->handle, last_fn, last_ret);

// failed:
   mc_pcmcia_release(link);
   link->state &= ~DEV_CONFIG_PENDING;

}


//----------------------------------------------------------------------------//
// mc_pcmcia_release()                                                        //
// After a card is removed, mc_pcmcia_release() will unregister the device,   //
// and release the PCMCIA configuration. If the device is still open,         //
// this will be postponed until it is closed.                                 //
//                                                                            //
//----------------------------------------------------------------------------//
void mc_pcmcia_release(dev_link_t *link)
{
   PK_DBG("mc_pcmcia_release(0x%p)\n", link);

   mc_pcmcia_task_stop();

   if(link->open)
   {
      PK_DBG("release postponed, still open");
      link->state |= DEV_STALE_CONFIG;
      return;
   }

   if(link->win)
   {
      pcmcia_release_window(link->win);
   }    
   
   //----------------------------------------------------------------
   // release the open ports
   //
   release_region(link->io.BasePort1, link->io.NumPorts1);
   free_irq(link->irq.AssignedIRQ, link->priv);
      
   pcmcia_release_configuration(link->handle);
   
   if(link->io.NumPorts1)
   {
      pcmcia_release_io(link->handle, &link->io);
   }
   
    
   link->state &= ~DEV_CONFIG; 
}


//----------------------------------------------------------------------------//
// mc_pcmcia_event()                                                          //
// The card status event handler.  Mostly, this schedules other stuff to run  //
// after an event is received. When a CARD_REMOVAL event is received, we      //
// immediately set a private flag to block future accesses to this device.    //
// All the functions that actually access the device should check this flag   //
// to make sure the card is still present.
//----------------------------------------------------------------------------//
static int mc_pcmcia_event(event_t event, int priority, 
                           event_callback_args_t *args)
{
   dev_link_t *      link  = args->client_data;
   _TsMcPcmciaInfo * dev   = link->priv;
    
   PK_DBG("mc_pcmcia_event()");
    
   switch (event)
   {
      case CS_EVENT_CARD_REMOVAL:
         PK_DBG("CS_EVENT_CARD_REMOVAL");
         link->state &= ~DEV_PRESENT;
         if(link->state & DEV_CONFIG)
         {
            mc_pcmcia_release(link);
         }
         break;

      case CS_EVENT_CARD_INSERTION:
         PK_DBG("CS_EVENT_CARD_INSERTION");
         link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
         mc_pcmcia_config(link);
         break;

      case CS_EVENT_PM_SUSPEND:
         PK_DBG("CS_EVENT_PM_SUSPEND");
         link->state |= DEV_SUSPEND;
         //--- Fall through... --------

      case CS_EVENT_RESET_PHYSICAL:
         PK_DBG("CS_EVENT_RESET_PHYSICAL");
         // Mark the device as stopped, to block IO until later
         dev->stop = 1;
         if(link->state & DEV_CONFIG)
         {
            pcmcia_release_configuration(link->handle);
         }
         break;

      case CS_EVENT_PM_RESUME:
         PK_DBG("CS_EVENT_PM_RESUME");
         link->state &= ~DEV_SUSPEND;
         /* Fall through... */

      case CS_EVENT_CARD_RESET:
         PK_DBG("CS_EVENT_CARD_RESET");
         if(link->state & DEV_CONFIG)
         {
            pcmcia_request_configuration(link->handle, &link->conf);
         }
         dev->stop = 0;
         /*
         In a normal driver, additional code may go here to restore
         the device state and restart IO.
         */
         break;
   }

   return 0;
}


/*----------------------------------------------------------------------------*\
** Initialization structures / functions                                      **
**                                                                            **
\*----------------------------------------------------------------------------*/

static struct pcmcia_driver mc_pcmcia_driver = {
   .owner	= THIS_MODULE,
   .drv		= {
      .name = "can_mc_pcmcia",
   },
   .attach	= mc_pcmcia_attach,
   .detach	= mc_pcmcia_detach,
};


//----------------------------------------------------------------------------//
// init_mc_pcmcia_cs()                                                        //
//                                                                            //
//----------------------------------------------------------------------------//
static int __init init_mc_pcmcia_cs(void)
{
   int   slStatusT;
   
   PK_INF("Init driver for MicroControl PCMCIA CAN card ..");
   
   //----------------------------------------------------------------
   // register PCMCIA driver / services and assign
   // the resources for the PCMCIA CAN interface
   //
   slStatusT = pcmcia_register_driver(&mc_pcmcia_driver);
   
   //----------------------------------------------------------------
   // start the CANpie driver for the CAN interface
   //
   mc_pcmcia_can_register(&auwMcPcmciaPortS[0]);
   

   return(slStatusT);
}


//----------------------------------------------------------------------------//
// exit_mc_pcmcia_cs()                                                        //
//                                                                            //
//----------------------------------------------------------------------------//
static void __exit exit_mc_pcmcia_cs(void)
{
   PK_INF("Remove driver for MicroControl PCMCIA CAN card ..");
   
   mc_pcmcia_can_unregister();
   
   pcmcia_unregister_driver(&mc_pcmcia_driver);
}


//----------------------------------------------------------------------------//
// module_init()                                                              //
//                                                                            //
//----------------------------------------------------------------------------//
module_init(init_mc_pcmcia_cs);


//----------------------------------------------------------------------------//
// module_exit()                                                              //
//                                                                            //
//----------------------------------------------------------------------------//
module_exit(exit_mc_pcmcia_cs);


