//****************************************************************************//
// File:          mc_pcmcia_cs.h                                              //
// Description:   CANpie hardware abstraction layer (HAL)                     //
// Author:        Uwe Koppe                                                   //
// e-mail:        koppe@microcontrol.net                                      //
//                                                                            //
//============================================================================//
// This program is free software; you can redistribute it and/or modify       //
// it under the terms of the GNU Lesser General Public License as published   //
// by the Free Software Foundation; either version 2.1 of the License, or     //
// (at your option) any later version.                                        //
//============================================================================//
//                                                                            //
// Date        History                                                        //
// ----------  -------------------------------------------------------------- //
// 29.07.2003  Initial version                                                //
//                                                                            //
//****************************************************************************//


//------------------------------------------------------------------------------
// CVS version information:
// $Id: mc_pcmcia_cs.h,v 1.1 2005/06/01 21:42:54 microcontrol Exp $
//------------------------------------------------------------------------------


#include <pcmcia/version.h>
#include <pcmcia/cs_types.h>
#include <pcmcia/cs.h>
#include <pcmcia/cistpl.h>
#include <pcmcia/cisreg.h>
#include <pcmcia/ds.h>




#ifdef  CP_DEBUG
extern unsigned short uwMcPcmciaDebugG;
#endif

#define  CP_DRV_NAME    "can_mc_pcmcia"


//---------------------------------------------------------
// PK_DBG: print kernel debug stuff
//
#ifdef  CP_DEBUG

#undef  PK_DBG
#define PK_DBG(fmt, args...)  printk(KERN_DEBUG "can_mc_pcmcia: " fmt "\n", ##args)

#else
#define PK_DBG(fmt, args...)  
#endif

//---------------------------------------------------------
// PK_INF: print kernel information
//
#undef  PK_INF
#define PK_INF(fmt, args...)  printk(KERN_INFO  "can_mc_pcmcia: " fmt "\n", ##args)


//---------------------------------------------------------
// PK_ERR: print kernel error
//
#undef  PK_ERR
#define PK_ERR(fmt, args...)  printk(KERN_ERR   "can_mc_pcmcia: " fmt "\n", ##args)


/*
#define CS_CHECK(fn, args...) \
while ((last_ret=CardServices(last_fn=(fn),args))!=0) goto cs_failed
*/

/*
#define CFG_CHECK(fn, args...) \
if (CardServices(fn, args) != 0) goto next_entry
*/

/*====================================================================*/


struct mc_pcmcia_info_t {
   dev_link_t  link;
   dev_node_t  node;
   int         stop;       // driver is stopped
   int         irg;        // IRQ number
   int         port;       // port base number
};

typedef struct mc_pcmcia_info_t  _TsMcPcmciaInfo;

/*====================================================================*/

int  mc_pcmcia_can_register(unsigned short * ptsPortArrayV);

int  mc_pcmcia_can_unregister(void);

int  mc_pcmcia_reset(void);

void mc_pcmcia_set_info(struct mc_pcmcia_info_t *);

void mc_pcmcia_task_start(void);

void mc_pcmcia_task_stop(void);

